require 'gosu'
require_relative 'piece'
require_relative 'joueur'

class Niveau

def initialize(fenetre, joueur, donnees_niveau)
  @fenetre = fenetre
  @pieces = []
  @joueur = joueur
  @donnees_niveau = donnees_niveau
  @total_lignes = 0
  @total_colonnes = 0
  @sortie = false
  if @donnees_niveau
    @total_lignes = @donnees_niveau.length
    if @total_lignes > 0
      @total_colonnes = @donnees_niveau.first.length
    end
    config_niveau
  end
end

def config_niveau
  @donnees_niveau.each_with_index do |donnee_ligne, ligne|
    colonne = 0
    donnee_ligne.each_char do |type_cell|
      piece = Piece.new(@fenetre, colonne, ligne, type_cell)
      # Change le comportement selon la nature de type_cell
      if piece.est_depart?
        @joueur.deplacer_vers(colonne, ligne)
      end
      @pieces.push(piece)
      colonne += 1
    end
  end
end

def button_down(id)
  if niveau_fini?
    return
  end
  colonne_delta = 0
  ligne_delta = 0
  if id == Gosu::KbLeft
    colonne_delta = -1
  elsif id == Gosu::KbRight
    colonne_delta = 1
  end
  if id == Gosu::KbUp
    ligne_delta = -1
  elsif id == Gosu::KbDown
    ligne_delta = 1
  end
  if deplacement_valide?(@joueur, colonne_delta, ligne_delta)
    @joueur.deplacer_de(colonne_delta, ligne_delta)
    piece = lit_piece(@joueur.colonne, @joueur.ligne)
    if piece.est_sortie?
      @sortie = true
      piece.cache!
    else
      @joueur.trouve(piece)
    end
  end
end

def lit_piece(colonne, ligne)
  if colonne < 0 || colonne >= @total_colonnes || ligne < 0 || ligne >= @total_lignes
    nil
  else
    @pieces[ligne * @total_colonnes + colonne]
  end
end

def deplacement_valide?(joueur, colonne_delta, ligne_delta)
  destination = lit_piece(joueur.colonne + colonne_delta, joueur.ligne + ligne_delta)
 if destination && destination.piece_peut_etre_entree?
   true
  else
    false
  end
end

def niveau_fini?
  @sortie
end

  def update
  end

def draw
  @pieces.each do |piece|
    piece.draw
  end
  @joueur.draw
end

end
