require 'gosu'

class Piece

TYPE_JOUEUR = 'J'
TYPE_DEPART = 'D'
TYPE_SORTIE = 'S'
TYPE_TRESOR = 'T'
TYPE_VIDE = "."
LARGEUR = 32
HAUTEUR = 32

attr_reader :ligne, :colonne, :type

def initialize(fenetre, colonne, ligne, type)
  @@couleurs ||= {rouge: Gosu::Color.argb(0xaaff0000), vert: Gosu::Color.argb(0xaa00ff00), or: Gosu::Color.argb(0xaaffff00), bleu: Gosu::Color.argb(0xaa0000ff)}
  @@font ||= Gosu::Font.new(24)
  @@fenetre ||= fenetre
  @ligne = ligne
  @colonne = colonne
  @type = type
  @cache = false
end

def draw
  if piece_est_dessinee? && !cache?
    x1 = @colonne * LARGEUR
    y1 = @ligne * HAUTEUR
    x2 = x1 + LARGEUR
    y2 = y1
    x3 = x2
    y3 = y2 + HAUTEUR
    x4 = x1
    y4 = y3
    c = couleur
    @@fenetre.draw_quad(x1, y1, c, x2, y2, c, x3, y3, c, x4, y4, c, 2)
    x_centre = x1 + (LARGEUR / 2)
    x_texte = x_centre - @@font.text_width("#{@type}") / 2
    y_texte = y1 + 4
    @@font.draw("#{@type}", x_texte, y_texte, 1)
  end
end

def couleur
  if est_joueur?
    @@couleurs[:rouge]
  elsif est_sortie?
    @@couleurs[:vert]
  elsif est_tresor?
    @@couleurs[:or]
  else
    @@couleurs[:bleu]
  end
end

def deplacer_vers(colonne, ligne)
  @colonne = colonne
  @ligne = ligne
end

def deplacer_de(colonne_delta, ligne_delta)
  deplacer_vers(@colonne + colonne_delta, @ligne +ligne_delta)
end

def est_tresor?
  @type == TYPE_TRESOR
end
def est_depart?
  @type == TYPE_DEPART
end
def est_sortie?
  @type == TYPE_SORTIE
end
def est_joueur?
  @type == TYPE_JOUEUR
end
def est_vide?
  @type == TYPE_VIDE || @type == ' '
end

def cache?
  @cache
end
def cache!
  @cache = true
end

def vide_piece
  @type = TYPE_VIDE
end

def piece_est_dessinee?
  !est_vide? && !est_depart?
end
def piece_peut_etre_entree?
  est_vide? || est_depart? || est_tresor? || est_sortie?
end

end

