require 'gosu'
require_relative 'cellule'

class Grille
  include Enumerable

def initialize(fenetre, colonnes, lignes, graines=nil)
  @fenetre = fenetre
  @total_lignes = lignes
  @total_colonnes = colonnes
  @plateau = config_grille
  semis(@plateau, graines)
end

def config_grille
  grille = []
  @total_lignes.times do |ligne|
    cellules = []
    @total_colonnes.times do |colonne|
      cellules << Cellule.new(@fenetre, false, colonne, ligne)
    end
    grille << cellules
  end
  grille
end

def semis(plateau, graines)
  if graines.nil? || graines.empty?
    40.times do
      plateau[rand(@total_lignes)][rand(@total_colonnes)].vivante!
    end
    else
      graines.each do |x,y|
        cellule(x,y).vivante!
    end
  end
end

def cellule(x, y)
  if @plateau[y]
    @plateau[y][x]
  else
    nil
  end
end

def each
  @total_lignes.times do |ligne|
    @total_colonnes.times do |colonne|
      yield cellule(colonne, ligne)
    end
  end
end

def position_chaque_cellule
  @total_lignes.times do |ligne|
    @total_colonnes.times do |colonne|
      yield colonne, ligne
    end
  end
end

def delavie?
  none? do |cellule|
    cellule.vivante?
  end
end

def lavie
  cellules_vivantes = []
  position_chaque_cellule do |x,y|
    cellules_vivantes << [x,y] if cellule(x,y).vivante?
  end
  cellules_vivantes
end

def cellules_voisines(x, y)
  cellules = []
  (y - 1).upto(y + 1) do |ligne|
    (x - 1).upto(x + 1) do |colonne|
      next if ligne < 0 || ligne >= @total_lignes
      next if colonne < 0 || colonne >= @total_colonnes
      next if x == colonne && y == ligne
      cellules << cellule(colonne,ligne)
    end
  end
  cellules.compact
end

def ==(autre)
  self.lavie == autre.lavie
end

def draw
  each do |grille_cell|
    if grille_cell
      grille_cell.draw
    end
  end
end

end
