require 'gosu'
require_relative 'grille'
require_relative 'cellule'

class Jeu

  GENERATION_FREQUENCE = 100 # en millisecondes
  SEMENCE = [[11,10],[11,11],[11,12]]
    SEMENCE_ALEA = []
  SEMENCE_DOUCE = [[1,0],[2,1],[0,2],[1,2],[2,2]]
  SEMENCE_FORTE = [[30,19],[30,20],[30,21],[29,17],[30,17],[31,17]]
  SEMENCE_EXTREME = [[12,12],[13,12],[14,12],[16,12],[12,13],[15,14],[16,14],[13,15],[14,15],[16,15],[12,16],[14,16],[16,16]]
  LISTE_GRAINES = [SEMENCE_ALEA, SEMENCE, SEMENCE_DOUCE, SEMENCE_FORTE, SEMENCE_EXTREME]
  GRILLE_LARGEUR = 80
  GRILLE_HAUTEUR = 80

def initialize(fenetre, generations, sim)
  @fenetre = fenetre
  graines = LISTE_GRAINES[sim - 1]
  @grille = Grille.new(@fenetre, GRILLE_LARGEUR, GRILLE_HAUTEUR, graines)
  @font = Gosu::Font.new(32)
  @heure_actuelle = @heure_depart = Time.now.to_i
  @derniere_actu = 0
  @generation = 0
  @max_generations = generations.to_i
  @message_etat = "Terminé"
end

def simulation_finie?
  (@max_generations > 0) && (@generation >= @max_generations)
end

def update
  cette_heure = Gosu::milliseconds()
  if (cette_heure - @derniere_actu > GENERATION_FREQUENCE && (@max_generations == 0 || @generation < @max_generations))
    nouv_quad = evolution
    @generation += 1
  if nouv_quad.delavie?
    @message_statut = "La vie a disparue"
    @max_generations = @generation
  elsif nouv_quad == @grille
    @message_statut = "La vie est gelée"
    @max_generations = @generation
  end
    @grille = nouv_quad
    @derniere_actu = cette_heure
    @heure_actuelle = Time.now.to_i
  end
end

def evolution
  vie = []
  @grille.position_chaque_cellule do |x,y|
    if determine_evol(x, y)
      vie << [x, y]
    end
  end
  Grille.new(@fenetre, GRILLE_LARGEUR, GRILLE_HAUTEUR, vie)
end

def determine_evol(x, y)
  cellule = @grille.cellule(x, y)
  voisines = @grille.cellules_voisines(x, y)
  score = 0
  voisines.each {|n| score += n.points_vie}
  (cellule.vivante? && score >= 2 && score <= 3) ||
  (score == 3)
end


  def draw
    @grille.draw
   affiche_infos
  end

def affiche_infos
  if simulation_finie?
    @font.draw("Simulation terminée !", 200, 150, 10, 2, 2)
    @font.draw("#{@message_statut} en #{@generation} générations après #{temps_en_secondes} secondes!", 110, 300, 10)
  else
    @font.draw("Temps : #{temps_en_secondes}", 4, 2, 10)
    @font.draw("Génération : #{@generation}", 540, 2, 10)
  end
end

def temps_en_secondes
  @heure_actuelle - @heure_depart
end

end

